varying highp vec4 var_position;
varying mediump vec3 var_normal;
varying mediump vec2 var_texcoord0;
varying mediump vec4 var_light;

uniform lowp sampler2D tex0;
uniform lowp vec4 tint;
uniform lowp vec4 light_color;

// Fresnel effect factors
uniform highp float fresnel_bias;
uniform highp float fresnel_scale;
uniform highp float fresnel_power;

void main()
{
    vec4 baseColor = texture2D(tex0, var_texcoord0.xy) * tint; // Base paint color

    // Ambient light component
    vec3 ambientLight = baseColor.xyz * vec3(0.3, 0.2, 0.4);// * light_color.xyz; // Ambient light to simulate environmental lighting

    // Normalize vectors
    vec3 norm = normalize(var_normal);
    vec3 lightDir = normalize(var_light.xyz - var_position.xyz);

    // Diffuse lighting calculation
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diff * baseColor.rgb;

    // Specular highlight for glossiness (Blinn-Phong model)
    vec3 viewDir = normalize(-var_position.xyz); // Assuming the view position is at the origin
    vec3 halfDir = normalize(lightDir + viewDir);
    float spec = pow(max(dot(norm, halfDir), 0.0), 32.0); // Shininess factor
    vec3 specular = vec3(0.0001) * spec; // Reduced intensity for the clear coat reflection

    // Apply Fresnel effect to reduce specular at grazing angles
    float fresnel = fresnel_bias + fresnel_scale * pow(2.0 - max(dot(viewDir, norm), 0.0), fresnel_power);
    specular *= fresnel;

    // Metallic flake effect (simplified without view position)
    float flake = pow(max(dot(halfDir, norm), 0.0), 2.5) * spec;
    vec3 flakeColor = vec3(1.0, 1.0, 1.0) * flake; // Flake sparkle effect

    // Combine all components
    vec3 finalColor = ambientLight + diffuse + specular + flakeColor;
    gl_FragColor = vec4(finalColor, baseColor.a); // Using base color's alpha for transparency
}
