varying highp vec4 var_position;
varying mediump vec3 var_normal;
varying mediump vec2 var_texcoord0;
varying mediump vec4 var_light;

uniform lowp sampler2D tex0;
uniform lowp vec4 tint;
uniform lowp vec4 light_color;

// Fresnel effect factors
uniform highp float fresnel_bias;
uniform highp float fresnel_scale;
uniform highp float fresnel_power;

void main()
{
    vec4 baseColor = texture2D(tex0, var_texcoord0.xy) * tint; // Base paint color

    // Ambient light component
    vec3 ambientLight = baseColor.xyz * vec3(0.7) * light_color.xyz; // Ambient light to simulate environmental lighting

    // Normalize vectors
    vec3 norm = normalize(var_normal);
    vec3 lightDir = normalize(var_light.xyz - var_position.xyz);

    // Diffuse lighting calculation
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diff * baseColor.rgb;

    // Specular highlight for matte glossiness (Blinn-Phong model)
    vec3 viewDir = normalize(-var_position.xyz); // Assuming the view position is at the origin
    vec3 halfDir = normalize(lightDir + viewDir);
    float spec = pow(max(dot(norm, halfDir), 0.0), 8.0); // Increase the exponent for a broader, softer highlight
    vec3 specular = vec3(0.00005) * spec; // Reduce intensity for a more subtle reflection

    // Apply Fresnel effect to reduce specular at grazing angles
    float fresnel = fresnel_bias + fresnel_scale * pow(1.0 - max(dot(viewDir, norm), 0.0), fresnel_power);
    specular *= fresnel;

    // Combine all components
    vec3 finalColor = ambientLight + diffuse + specular;
    gl_FragColor = vec4(finalColor, baseColor.a); // Using base color's alpha for transparency
}
