varying highp vec4 var_position;
varying mediump vec3 var_normal;
varying mediump vec2 var_texcoord0;
varying mediump vec4 var_light;

uniform lowp sampler2D tex0;
uniform lowp vec4 tint;
uniform lowp vec4 light_color;

void main()
{
    vec4 baseColor = texture2D(tex0, var_texcoord0.xy) * tint; // Base paint color

    // Ambient light component
    vec3 ambientLight = baseColor.xyz * vec3(0.7) * light_color.xyz; // Ambient light to simulate environmental lighting

    // Diffuse lighting calculation
    vec3 lightDir = normalize(var_light.xyz - var_position.xyz);
    float diff = max(dot(var_normal, lightDir), 0.0);
    vec3 diffuse = diff * baseColor.rgb;

    // Specular highlight for glossiness
    vec3 reflectDir = reflect(-lightDir, var_normal);
    float spec = pow(max(dot(lightDir, reflectDir), 0.0), 32.0); // Shininess factor
    vec3 specular = vec3(0.0001) * spec; // Reduced intensity for the clear coat reflection

    // Metallic flake effect (simplified without view position)
    float flake = pow(max(dot(reflectDir, var_normal), 0.0), 2.5) * spec;
    vec3 flakeColor = vec3(1.0, 1.0, 1.0) * flake; // Flake sparkle effect

    // Combine all components
    vec3 finalColor = ambientLight + diffuse + specular + flakeColor;
    gl_FragColor = vec4(finalColor, baseColor.a); // Using base color's alpha for transparency
}
